/* 	ClassTemplate.java
 * 
 * 	Created on: 04-09-2002
 * 	Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2002-2004, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]				[What]
 * 09/05/2002	dennis@brains2b.nl	added possibility to create Inner classes
 * 10/28/2002	dennis@brains2b.nl	Moved methods to Template interface and TemplateHelper 
 * 11/06/2002	dennis@brains2b.nl	Implemented imports through TreeSet,
 * 									Expanded createMember
 * 11/15/2002		"		"		Added possibility to add comment to class
 * 19-02-2003		"		"		improvements to the layout of the written class, orders the methods
 * 03-11-2003 dennis@brains2b.nl	Changes for naming parameters in MethodTemplate 
 * 15-02-2004 dennis@brains2b.nl	Added completeFile method to return the name of the actually created file
 * 03-02-2005 dennis@brains2b.nl	Added addInterface method
 * 14-03-2005 dennis@brains2b.nl	setName implemented with renaming constructors, fixed unicity for internal hashkeys for 
 * 								Constructor
 * 11-04-2006 dennis@brains2b.nl	Write a blank line between package and import
 * 18-06-2006 dennis@brains2b.nl	Don't allow double interfaces
 * 28-11-2006 dennis@brains2b.nl    Allow for modifiers on members
 */
package org.brains2b.typewriter;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeSet;

import org.brains2b.util.JavaHelper;

/** Template for creating a complete class file
 * <p>call the <code<writeClass(String)</code> to write
 * a class to a specified path, including the correct indention
 * packagename,imports,etc.
 * <p>This ClassTemplate can also be used to create Inner classes into
 * another ClassTemplate, through <code>createInnerClass(String, String, String[])</code>
 * <p>The class written will order the imports alphabetically, orders the method
 * starting at public down to private, ordering methods alphabetically. Same for
 * the method
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.32.2 [28-11-2006]
 * @see #createInnerClass(String, String, String[])
 */
public class ClassTemplate implements Template {

	private CommentTemplate m_comment;
	private ArrayList m_imports;
	private ArrayList m_interfaces;
	private HashMap m_methods;
	private HashMap m_members;
	private HashMap m_innerClasses;
	
	private String m_name;
	private String m_package;
	private String m_superclass;
	private File completeFile;
	
	private ClassTemplate m_parent;
	
	/**
	 * Constructor for creating a ClassTemplate for an Innerclass
	 * @param parent ClassTemplate, the class which should contain this inner class, if set to
	 * <code>null</code> this will become a toplevel class
	 * @param name String, the name of this class
	 * @param superclass String, fully qualified class name for supper class, set to <code>null</code> if super class
	 * should be java.lang.Object. <i>This is not explicitly written into the class file</i> 
	 * @param interfaces String[], an array containing fully qualified class names for the interfaces,set to <code>null</code>
	 * if this class does not implement any interfaces
	 * this method should implement 
	 */
	protected ClassTemplate(ClassTemplate parent,String name, String superclass, String[] interfaces) {	
		super();
		m_parent=parent;
		m_name=name;
		m_superclass=TypeWriterHelper.getClassName(superclass);
		
		m_methods = new HashMap();
		m_members = new HashMap();
		m_innerClasses = new HashMap();
		m_imports = new ArrayList();
		m_interfaces = new ArrayList();
		
		addImport(superclass);
		if (interfaces!=null)
		for (int i=0;i<interfaces.length;i++) {
			addInterface(interfaces[i]);
		}
	}		
		
	/**
	 * Constructor for creating a ClassTemplate
	 * @param name String, the name of this class, will be used to make the filename
	 * @param superclass String, fully qualified class name for supper class, set to <code>null</code> if super class
	 * should be java.lang.Object. <i>This is not explicitly written into the class file</i>
	 * @param interfaces String[], an array containing fully qualified class names for the interfaces this method should implement,
	 * set to <code>null</code> if this class does not implement any interfaces
	 *  
	 */
	public ClassTemplate(String name, String superclass, String[] interfaces) {
		this((ClassTemplate) null,name,superclass,interfaces);
	}
	
	/**
	 * Constructor for creating a ClassTemplate
	 * @param name String, the name of this class, will be used to make the filename
	 * @param superclass String, fully qualified class name for supper class, set to <code>null</code> if super class
	 * should be java.lang.Object. <i>This is not explicitly written into the class file</i>
	 */
	public ClassTemplate(String packagename,String name, String superclass, String[] interfaces) {
		this((ClassTemplate) null,name,superclass,interfaces);
		setPackage(packagename);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void write(PrintWriter pw, int indent) {
		String spc=TypeWriterHelper.spaces(indent*4);
		//write the file comment
		if (m_comment!=null) m_comment.write(pw,indent);
		
		boolean emptyLine=false;
		//write the package
		if (m_package!=null && m_parent==null) {
			pw.print("package ");
			pw.print(m_package);
			pw.println(";");
			pw.println();
			emptyLine=true;
		}
		
		// write the imports
		Collections.sort(m_imports);
		for (Iterator it=m_imports.iterator();it.hasNext();) {
			pw.print("import ");
			pw.print(it.next());
			pw.println(";");
		}
		if (!m_imports.isEmpty()) {
			pw.println();
			emptyLine=true;
		}
	
		if (!emptyLine && m_parent==null) pw.println();
		
		//write the header
		pw.print(spc+"public class ");
		pw.print(m_name);
		if (m_superclass!=null) {
			pw.print(" extends ");
			pw.print(m_superclass);	
		}
		if (!m_interfaces.isEmpty()) {
			Collections.sort(m_interfaces);
			pw.print(" implements " );
			for (int i=0;i<m_interfaces.size();i++) {
				pw.print(m_interfaces.get(i));
				if (i<m_interfaces.size()-1) {
					pw.print(", ");
				}
			}
		}
		
		pw.println(" {");
		pw.println();
		//InnerClasses
		for (Iterator it = m_innerClasses.values().iterator();it.hasNext();) {
			((ClassTemplate) it.next()).write(pw,indent+1);
			pw.println();
		}				
		
		TreeSet set = new TreeSet(m_methods.values());
		for (Iterator it=set.iterator();it.hasNext();) {
			((MethodTemplate) it.next()).write(pw,indent+1);
			pw.println();
		}
		
		set=new TreeSet(m_members.keySet());
		for (Iterator it=set.iterator();it.hasNext();) {
			((MemberTemplate) m_members.get(it.next())).write(pw,indent+1);
			pw.println();
		}
		pw.println(spc+"}");
		
	}		
	
	/**
	 * get the filename of the completed file.
	 * This will only be non null if the writeClass(String) method has been called
	 * @return File, the File of the class which was just written
	 * @since 0.11
	 */
	public File getCompleteFile() {
		return completeFile;
		 	
	}
	
	/**
	 * write a class file with filename equals name+".java" to a given 
	 * path
	 * <p>If a package name is defined the appropriate directories are
	 * added to the path and if necessary created. The path supplied here
	 * should however exist   
	 * @param path String, the path to write the class file to, excluding directories
	 * which are contained in the package name
	 * @throws IOException, thrown if the file operation cannot complete
	 * normally
	 */
	public void writeClass(String path) throws IOException {
		if(m_name==null) return;
		File dir=getPackageDir(new File(path),getPackage());
		completeFile=new File(dir, m_name +".java");
		FileWriter fw=new FileWriter(completeFile);
		PrintWriter pw=new PrintWriter(fw);
		write(pw,0);		
		pw.close();
	}
	
	/**
	 * create an inner class template in this ClassTemplate
	 * @param name String, name of this inner class
	 * @param superclass String, fully qualified class name to use as super class
	 * for this inner class, if set to null the default java.lang.Object will be used
	 * @param interfaces String[], an array of fully qualified class names for the interfaces this
	 * inner class should implement, set to null if this class does not implement any
	 * interfaces
	 * @return ClassTemplate, the template to use with this innerclass 
	 */
	public ClassTemplate createInnerClass(String name, String superclass, String[] interfaces ) {	
		ClassTemplate result=( ClassTemplate) m_innerClasses.get(name);
		if (result==null) {
			result=new ClassTemplate(this,name,superclass, interfaces);
			m_innerClasses.put(name,result);
		}
		return result;	
	}
	
	/**
	 * create a comment template for this class file
	 * @return ContentTemplate, the template holding comments for this class
	 */
	public CommentTemplate createComment() {
		m_comment=new CommentTemplate();
		return m_comment;
	}
	
	/**
	 * create a constructor for this class template
	 * @param params String[], the types for the arguments to create a constructor
	 * with. The variable names for this constructor will be generated as arg0, arg1, etc.
	 * @return MethodTemplate, an instance of MethodTemplate where isConstructor is true
	 */
	public MethodTemplate createConstructor(String[] params) {
		MethodTemplate mth= new MethodTemplate(this,Modifier.PUBLIC,m_name,null,params);
		mth.setConstructor(true);
		m_methods.put(createSignature(m_name,params), mth);
		return mth;
	}			
	
	/**
	 * create a constructor for this class template
	 * @param params String[][], an array containing the types for the arguments to create a constructor
	 * with and the variable names used for the types, could be something like new String[] {{"int","String"},{"id","name"}}
	 * @return MethodTemplate, an instance of MethodTemplate where isConstructor is true
	 */
	public MethodTemplate createConstructor(String[][] params) {
			MethodTemplate mth= new MethodTemplate(this,Modifier.PUBLIC,m_name,null,params);
			mth.setConstructor(true);
			m_methods.put(createSignature(m_name,params[0]), mth);
			return mth;
	}		
	
	/**
	 * create a method with public access and no return value for the given name and with the parameters
	 * <p>If a method with the exact name and parameters already exist this method is returned instead of creating a new
	 * one
	 * @param name String, the name of this method, i.e. "setName"
	 * @param params String[], the types for the arguments to pass to the method
	 * with. The variable names for this constructor will be generated as arg0, arg1, etc.
	 * @return MethodTemplate
	 */
	public MethodTemplate createMethod(String name,String[] params) {
		return createMethod(Modifier.PUBLIC,name,null,params);
	}
	
	/**
	 * create a method with public access and the given type of return value for the given name and with the parameters
	 * <p>If a method with the exact name and parameters already exist this method is returned instead of creating a new
	 * one
	 * @param name String, the name of this method, i.e. "setName"
	 * @param returnType String, the type of return value, could be a class name or a primitive name, i.e. "int"
	 * @param params String[], the types for the arguments to pass to the method
	 * with. The variable names for this constructor will be generated as arg0, arg1, etc.
	 * @return MethodTemplate
	 */
	public MethodTemplate createMethod(String name,String returnType, String[] params) {
		return createMethod(Modifier.PUBLIC,name,returnType,params);
	}
	
	/**
	 * create a method with the given access, the given type of return value for the given name and with the parameters
	 * <p>If a method with the exact name and parameters already exist this method is returned instead of creating a new
	 * one
	 * @param modifiers int,one ore more access-modifiers from <code>Modifier</code>, i.e <code>Modifier.PRIVATE</code> 
	 * or <code>Modifier.PUBLIC+Modifier.STATIC+Modifier.FINAL</code>
	 * @param name String, the name of this method, i.e. "setName"
	 * @param returnType String, the type of return value, could be a class name or a primitive name, i.e. "int"
	 * @param params String[], the types for the arguments to pass to the method
	 * with. The variable names for this constructor will be generated as arg0, arg1, etc.
	 * @return MethodTemplate
	 * @see Modifier
	 */
	public MethodTemplate createMethod(int modifiers, String name, String returnType, String[] params) {	
		String mthName=createSignature(name,params);
		MethodTemplate result=( MethodTemplate) m_methods.get(mthName);
		if (result==null) {
			result=new MethodTemplate(this,modifiers,name,returnType, params);
			m_methods.put(mthName,result);
		}
		return result;	
	}
	
	/**
	 * create a method with the given access, the given type of return value for the given name and with the parameters
	 * <p>If a method with the exact name and parameters already exist this method is returned instead of creating a new
	 * one
	 * @param modifiers int,one ore more access-modifiers from <code>Modifier</code>, i.e <code>Modifier.PRIVATE</code> 
	 * or <code>Modifier.PUBLIC+Modifier.STATIC+Modifier.FINAL</code>
	 * @param name String, the name of this method, i.e. "setName"
	 * @param returnType String, the type of return value, could be a class name or a primitive name, i.e. "int"
	 * @param params String[][], an array containing the types for the arguments to create a constructor
	 * with and the variable names used for the types, could be something like new String[] {{"int","String"},{"id","name"}}
	 * @return MethodTemplate
	 * @see Modifier
	 */
	public MethodTemplate createMethod(int modifiers, String name, String returnType, String[][] params) {	
		String mthName=createSignature(name,params[0]);
		MethodTemplate result=( MethodTemplate) m_methods.get(mthName);
		if (result==null) {
			result=new MethodTemplate(this,modifiers,name,returnType, params);
			m_methods.put(mthName,result);
		}
		return result;	
	}
	
	/**
	 * create a private member variable in this ClassTemplate
	 * @param type String, type of the member variable, i.e. "String"
	 * @param name String, the name of a membervariable
	 * @param initialValue Object, a representation of the initial value, either a String
	 * or a Literal
	 * @return MemberTemplate, can be ignored while all the work is already done
	 * @see Literal
	 */
	public MemberTemplate createMember(String type,String name, Object initialValue) {
		MemberTemplate mem=(MemberTemplate) m_members.get(name);
		if (mem==null) {
			mem=new MemberTemplate(TypeWriterHelper.getClassName(type),name, initialValue);
			m_members.put(name,mem);
		}
		return mem; 
	}
	
	/**
	 * create a private member variable in this ClassTemplate
	 * @param type String, type of the member variable, i.e. "String"
	 * @param name String, the name of a membervariable
	 * @param initialValue Object, a representation of the initial value
	 *  * @param initialValue Object, the initial value for this member, supply a Literal
	 * object for a String or char that should be used as a Literal value
	 * seen as a String but as a Literal value. same as using (String,String,Literal)
	 * @return MemberTemplate, can be ignored while all the work is already done
	 * @see Literal
	 */
	public MemberTemplate createMember(int modifier,String type,String name, Object initialValue) {
		MemberTemplate mem=(MemberTemplate) m_members.get(name);
		if (mem==null) {
			mem=new MemberTemplate(modifier,TypeWriterHelper.getClassName(type),name, initialValue);
			m_members.put(name,mem);
		}
		return mem; 
	}
    
    public MemberTemplate createMember(String type,String name, Object initialValue, boolean literal) {
         return createMember(Modifier.PRIVATE, type,name,initialValue);
    }
	
	/**
	 * create a private member variable in this ClassTemplate without an initial value
	 * @param type String, type of the member variable, i.e. "String"
	 * @param name String, the name of a membervariable
	 * @return MemberTemplate, can be ignored while all the work is already done
	 */
	public MemberTemplate createMember(String type,String name) {
		return createMember(Modifier.PRIVATE, type,name,null);	
	}

   
    
	/** 
	 * Add a interface to this class by supplying its full class name
	 * @param className String, the class name of the interface to add
	 * @since TypeWriter 0.21
	 */
	public void addInterface(String className) {
		if (className==null) return;
		if (!m_interfaces.contains(className)) {
			m_interfaces.add(TypeWriterHelper.getClassName(className));
			addImport(className);
		}
	}
	
	/**
	 * add an import to this ClassTemplate if it has no parent 
	 * (is not an Innerclass).
	 * <p>imports are added if the className contains package information
	 * <p>The imports are backed by a TreeSet to sort the imports and prevent
	 * imports to be duplicated
	 * @param className String, the className to add to the import
	 * @see org.brains2b.typewriter.Template#addImport(java.lang.String)
	 */
	public void addImport(String className) {
		if (className==null || className.indexOf('.')==-1) return;
		if (getParent()!=null) {
			((ClassTemplate) getParent()).addImport(className);
		} else if (!m_imports.contains(className)){
			m_imports.add(className);
		}
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public Template getParent() {
		return m_parent;
	}

	
	/**
	 * {@inheritDoc}
	 */
	public void setParent(Template parent) {
		m_parent = (ClassTemplate) parent;
	}

	/**
	 * get the name of this class
	 * @return String the name of this class
	 */
	public String getName() {
		return m_name;
	}

    /**
     * get the package for this class
     * @return String, the package name 
     */
    public String getPackage() {
        return m_package;
    }

    /**
     * set the package name for this class
     * <p>It is assumed that this is a valid package name
     * @param p String, the package to set
     */
    public void setPackage(String p) {
        m_package = p;
    }
    
    /**
     * get the directories for the package and create them on the filesystem
     * if they are not yet present
     * @param fl File, the path to write the file to as supplied to <code>writeClass(String)</code>
     * @param name String, the package name
     * @return File, a File object describing the complete directory (supplied and package directory) to write this class file to    
     * @throws IOException, thrown if the supplied path does not exist or the directories for
     * the package do not exist and cannot be created
     */
    private File getPackageDir(File fl, String name) throws IOException {
    	if (!fl.exists()) { throw new FileNotFoundException(fl.getCanonicalPath() + " is not found on this system"); }
    	if (name==null || name.length()==0) return fl;
    	
    	final char separator=System.getProperty("file.separator").charAt(0);
    	name=name.replace('.',separator);
    	File result=new File(fl,name);		
    	if (result.exists() || result.mkdirs()) {
    		return result;
    	} 
    	throw new IOException("Creation of package "+ name +" at " + fl.getCanonicalPath() + "failed");
    	   		
    }

    /**
     * make a unique name from methodname and parameter types, used for sorting
     * and making sure this signature is unique
     * @param name String, the name of this method
     * @param params, array of parameter types passed to this method
     * @return String, a signature of this method
     */
	private String createSignature(String name ,  String[] params) {
		StringBuffer sb=new StringBuffer(name);
		if (params!=null && params.length>0) {
			for (int i=0;i<params.length;i++) {
				sb.append(JavaHelper.objectName(params[i]));
			}
		}
		return sb.toString();
	}
	
    /**
     * set the name of the class
     * @param name String, The class name / filename for this Class
     * @since TypeWriter 0.21
     */
    public void setName(String name) {
        if (!m_name.equals(name)) { // only reset if changed
	        Iterator it=m_methods.keySet().iterator();
	        while (it.hasNext()) { // rename the constructors
	            String key= (String) it.next();
	            MethodTemplate t= (MethodTemplate) m_methods.get(key);
	            if (key.startsWith(m_name)) {
	                key.replaceFirst(m_name,name);
	                t.setName(name);
	            }
	        }
	        m_name = name;
        }
    }
}
