/* 	ContentTemplate.java
 * 
 * 	Created on: Oct 21, 2002
 * 	Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2002, 2003, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]				[What]
 * 10/28/2002	dennis@brains2b.nl	Moved methods to Template interface and TemplateHelper
 */
package org.brains2b.typewriter;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

/** 
 * Abstract ContentTemplate which is the basis for most of what a java
 * method can contain
 * <p>Provides methods for creating contents for a method for conditions,
 * loops and exceptions
 * <p>Every class which extends this should implement it's own <code>write(PrintWriter,indent)</code>
 * method and call <code>super.write(PrintWriter,indent)</code> to write out the content of this
 * method
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10 [28-10-2002]
 */
public abstract class ContentTemplate implements Template {

	protected List m_content=new ArrayList();
	private Template m_parent;

	/**
	 * add a line to the content of this template.
	 * <p>A line can be anything, but make sure you end it with <code>;</code>
	 * if it needs to be a valid java statement
	 * <p>The line-end will be added by this Template
	 * @param text String, a line to add to the content
	 */
	public void addText(String text) {
			m_content.add(text);
	}

	/**
	 * create a LoopTemplate for a <code>for-next-loop</code> within this content
	 * @param condition String, the condition for this loop
	 * @return LoopTemplate
	 */
	public LoopTemplate createFor(String condition) {
		LoopTemplate lT=new LoopTemplate(LoopTemplate.FOR,condition);
		lT.setParent(this);
		m_content.add(lT);
		return lT;
	}
	
	/**
	 * create a LoopTemplate for a <code>while-loop</code> within this content
	 * @param condition String, the condition for this loop
	 * @return LoopTemplate
	 */
	public LoopTemplate createWhile(String condition) {
		LoopTemplate lT=new LoopTemplate(LoopTemplate.WHILE,condition);
		lT.setParent(this);
		m_content.add(lT);
		return lT;
	}
	
	/**
	 * create an IfTemplate for a <code>condition-statement</code> within this content
	 * @param condition String, the expression for this condition
	 * @return IfTemplate
	 */
	public IfTemplate createIf(String condition) {
		IfTemplate ifT=new IfTemplate(condition);
		ifT.setParent(this);
		m_content.add(ifT);
		return ifT;
	}
	
	/**
	 * create a TryTemplate for a <code>try-catch-block</code> within this content
	 * @param exceptionClassName String, the name of this Exception
	 * @return TryTemplate
	 */
	public TryTemplate createTry(String exceptionClassName) {
		TryTemplate tryT=new TryTemplate(exceptionClassName);
		tryT.setParent(this);
		m_content.add(tryT);
		return tryT;
	}
	
	/**
	 * overwrite this to implement a content types specific needs and
	 * call <code>super.write(PrintWriter,int)</code> to write out
	 * the content for this method.
	 * @see org.brains2b.typewriter.Template#write(java.io.PrintWriter, int)
	 */
	public void write(PrintWriter writer, int indent) {
		String spc=TypeWriterHelper.spaces(indent*4);
		for (int i=0;i<m_content.size();i++) {
			if (m_content.get(i) instanceof Template) {
				((Template) m_content.get(i)).write(writer,indent);
			} else {
				writer.print(spc);
				writer.println(m_content.get(i));
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public Template getParent() {
		return m_parent;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void setParent(Template parent) {
		m_parent = parent;
	}

	/**
	 * get the content for this template
	 * <p>The list will either contain Strings or other ContentTemplates
	 * @return Vector, a list of objects contained in this template, will always
	 * return a List even if there is no content.
	 */
	public List getContent() {
		return m_content;
	}

	/**
	 * set the content for this Template
	 * <p>The List should contain either contain Strings or other ContentTemplates
	 * @param content Vector, a list of objects contained in this template, if null is inserted
	 * the content of this ContentTemplate will be removed and getContent() will return
	 * an empty List, not null.
	 */
	public void setContent(List content) {
		if (content==null) {
			m_content.clear();
		} else {
			m_content = content;
		}
	}
	
	/**
	 * create a local instance for a class by calling its constructor
	 * with the given parameter and adds it to the content
	 * @param className String, the class to instantiate, could be the fully
	 * qualified className
	 * @param name String the variable name of the instantiated class
	 * @return String name, the name of the instance made, should be equal to name
	 * passed to this method
	 * @see #makeInstance(String, String, Object[])
	 */
	public String makeInstance(String className, String name) {
		return makeInstance(className,name,(Object[]) null);
	}
	
	/**
	 * create a local instance for a class by calling its constructor
	 * with the given parameter and adds it to the content
	 * @param className String, the class to instantiate, could be the fully
	 * qualified className
	 * @param name String the variable name of the instantiated class
	 * @param arg Object, a single argument passed to the constructor of
	 * the class defined by className
	 * @return String name, the name of the instance made, should be equal to name
	 * passed to this method
	 * @see #makeInstance(String, String, Object[])
	 */
	public String makeInstance(String className, String name, Object arg) {
		return makeInstance(className,name,new Object[] {arg});
	}

	/**
	 * Create a local instance for a class by calling its constructor
	 * with the given parameters and adds it to the content
	 * @param className String, the class to instantiate, could be the fully
	 * qualified className
	 * @param name String the variable name of the instantiated class
	 * @param args Object[], an array of arguments passed to the constructor of
	 * the class defined by className
	 * @return String name, the name of the instance made, should be equal to name
	 * passed to this method
	 */
	public String makeInstance(String className, String name, Object[] args) {
		String objName=TypeWriterHelper.getClassName(className);
		StringBuffer sb=new StringBuffer();
		sb.append(objName);
		sb.append(" ").append(name).append(" = ");
		sb.append("new ").append(objName);
		if (args==null) {
			sb.append("();");
		} else {
			sb.append("( ");
			for (int i=0;i<args.length;i++) {
				if (args[i]==null) {
					sb.append("null");
				} else {
					sb.append(String.valueOf(args[i]));
				}
				if (i<args.length-1) {
					sb.append(", ");
				} 
			}
			sb.append(" );");
		}
		m_content.add(sb.toString());
		return name;
	}
	
	/* (non-Javadoc)
	 * @see org.brains2b.typewriter.Template#addImport(java.lang.String)
	 */
	public void addImport(String name) {
		if (m_parent!=null)
			m_parent.addImport(name);
	}

}
