/* 	ExceptionTemplate.java
 *	Copyright (c) 2004, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: May 11, 2003
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.typewriter;

import java.io.PrintWriter;

/** 
 * Template for creating a "} catch ([Exception] [ex]) {" statement in a template  
 * <p>This Template will make an variable name out of the Exception itself
 * by taking all capital letters in an exception name and putting them one after another
 * and finally adding ex to it. So MalformedUrlException becomes muex;
 * <p>If Exception itself is used ex will be the variable name, if Throwable is used t will
 * be the variable name
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.11 22-08-2004
 * @see org.brains2b.typewriter.TryTemplate#createException(String)
 * @see org.brains2b.typewriter.ContentTemplate#createTry(String)
 */
public class ExceptionTemplate extends ContentTemplate {

	private String m_exception;
	
	/**
	 * Constructor for ExceptionTemplate.
	 * @param exceptionClassName String, the name of the Exception to use. If this
	 * name is fully qualified it will be added to the import statements
	 * @see ContentTemplate#addImport(String)
	 */
	public ExceptionTemplate(String exceptionClassName) {
		super();
		addImport(exceptionClassName);
		m_exception=TypeWriterHelper.getClassName(exceptionClassName);
	}	
	

	/**
	 * {@inheritDoc}
	 */
	public void write(PrintWriter writer, int indent) {
		if (m_exception==null || m_exception.length()==0) {
			throw new TypeWriterException("Cannot create ExceptionTemplate without an Exception"); 
		}
		String spc=TypeWriterHelper.spaces(indent*4);
		writer.print(spc);
		writer.print("} catch (");
		writer.print(m_exception);
		writer.print(" ");
		writer.print(getExceptionName(m_exception));
		writer.println(") {");

		super.write(writer,++indent);
		
	}

	/**
	 * makes a variable name to use in the code of the catch statement
	 * <p>by taking all capital letters in an exception name and putting them one after another
	 * and finally adding ex to it. So MalformedUrlException becomes muex;
	 * <p>If Exception itself is used ex will be the variable name, if Throwable is used t will
	 * be the variable name
	 * @param exception
	 * @return
	 */
	private String getExceptionName(String exception) {
		StringBuffer sb=new StringBuffer();
		int idx=exception.indexOf("Exception");
		if (idx!=-1) {
			if (idx>1)
			for (int i=0;i<idx;i++) {
				if (Character.isUpperCase(exception.charAt(i))) {
					sb.append(Character.toLowerCase(exception.charAt(i)));
				}
			}
			sb.append("ex");
		} else {
			sb.append("t");
		}
		return sb.toString();
	}

}
