/* 	IfTemplate.java
 *	Copyright (c) 2003-2006, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: May 11, 2003
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * @#History 
 * [When]       [Who]               [What] 
 * 19-okt-2005  dennis@brains2b.nl  Extra method to write short form of 'else if'
 * 11-06-2006	dennis@brains2b.nl	Made getCondition public
 */
package org.brains2b.typewriter;

import java.io.PrintWriter;

import org.brains2b.util.StringHelper;

/** 
 * Template for creating conditions and else statements in another
 * ContentTemplate
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.12.1 11-06-2006
 * @see ContentTemplate#createIf(String)
 */
public class IfTemplate extends ContentTemplate {

	private String m_condition;	
	private BlockTemplate m_elseT;
	
	/**
	 * Constructor for IfTemplate.
	 * @param condition String, an expression defining the condition for this statement
	 */
	public IfTemplate(String condition) {
		super();
		m_condition=condition;
	}
	
	/**
	 * <p>this implementation of write follows the rule that if 
	 * this statement has an else statement which only contains
	 * another if statement the statement should read <code>} else if (</code>
	 * instead of closing the else and starting with a new condition. 
	 * @see org.brains2b.typewriter.Template#write(java.io.PrintWriter, int)
	 */
	public void write(PrintWriter writer, int indent) {
        write(writer,indent,true);
    }
    
	/**
	 * write the if statement to the PrintWriter
	 * @param writer PrintWriter
	 * @param indent int, current indent
	 * @param firstLine boolean, false if this is a condition which is the only 
	 * content of an else branch, true if it is a regular condition
	 * @see org.brains2b.typewriter.Template#write(java.io.PrintWriter, int)
	 * @throws TypeWriterException, if the condition is not set or if it is empty
	 */
    public void write(PrintWriter writer, int indent, boolean firstLine) {
		if (StringHelper.isEmpty(m_condition)) throw new TypeWriterException(
			"cannot write an IfTemplate without a condition"
		);
		String spc=TypeWriterHelper.spaces(indent*4);
		if (firstLine) writer.print(spc);
		writer.print("if(");
		writer.print(m_condition) ;
		writer.println(") {");
		
		super.write(writer,indent+1);
		if (m_elseT==null) {
			writer.println(spc+"}");
		} else {
			if (m_elseT.getContent().size()==1 && m_elseT.getContent().get(0) instanceof IfTemplate) {
				writer.print(spc+"} else ");
				((IfTemplate) m_elseT.getContent().get(0)).write(writer,indent,false);
			} else {
				writer.print(spc+ "}");
				m_elseT.write(writer, indent);
			}
		}	
	}

	/**
	 * create an else statement for this condition
	 * @return IfTemplate, a new instance of this IfTemplate used
	 * for it's else statement
	 */
	public BlockTemplate createElse() {
		m_elseT=new BlockTemplate("else");
		m_elseT.setParent(this);
		return m_elseT;
	}

	/**
	 * set the expression to evaluate for this condition
	 * @param condition String, the expression
	 */
	public void setCondition(String condition) {
		m_condition = condition;
	}

	/**
	 * get the condition.
	 * @return String
	 */
	public String getCondition() {
		return m_condition;
	}

}
