/* 	MemberTemplate.java
 *	Copyright (c) 2004, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Sept 19, 2002
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]				[What]
 * 10/28/2002	dennis@brains2b.nl	Moved methods to Template interface and TemplateHelper
 * 11/06/2002		"		"		Add literal option to intialization
 * 04-05-2006	dennis@brains2b.nl	made accessors for type public
 */
package org.brains2b.typewriter;

import java.io.PrintWriter;
import java.lang.reflect.Modifier;

/** Template for creating a Member in a class
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10.1 [04-05-2006]
 */
public class MemberTemplate implements Template {

	private ClassTemplate m_parent;
	private int m_modifiers;
	private String m_type;
	private String m_name;
	private Object m_initial;
	/**
	 * Constructor
	 * <p>Make a private member with the given name and type
	 * @param type String, representation of the type 
	 * @param name String, the name of the method
	 */
	public MemberTemplate(String type, String name) {
		this(Modifier.PRIVATE,type,name,null);
	}

	/**
	 * Constructor
	 * <p>Make a private member with the given name and type
	 * @param type String, representation of the return value
	 * @param name String, the name of the method
	 * @param initialValue Object, the initial value for this method, supply a Literal
	 * object for a String or char that should be used as a Literal value
	 */
	public MemberTemplate(String type, String name, Object initialValue) {
		this(Modifier.PRIVATE,type,name,initialValue);
	}

	/**
	 * Constructor
	 * <p>Make a member with the given name and type and with the given
	 * access.
	 * @param modifiers int, sum of the modifiers to use for this method
	 * @param type String, representation of the return value
	 * @param name String, the name of the method
	 */
	public MemberTemplate(int modifiers, String type, String name)  {
		this (modifiers,type,name,null);
	}
	
	/**
	 * Constructor
	 * <p>Make a member with the given name and type and with the given
	 * access and initial value.
	 * @param modifiers int, sum of the modifiers to use for this method
	 * @param type String, representation of the return value
	 * @param name String, the name of the method
	 * @param initialValue Object, the initial value for this method, supply a Literal
	 * object for a String or char that should be used as a Literal value
	 */
	public MemberTemplate(int modifiers, String type, String name,Object initialValue)  {
		super();
		if (type==null || type.length()==0) {
			throw new TypeWriterException("Member type must be supplied");	
		}
		if (name==null || name.length()==0) {
			throw new TypeWriterException("Member name cannot be empty");
		}
		m_modifiers=modifiers;
		m_type=type;
		m_name=name;
		m_initial=initialValue;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void write(PrintWriter pw, int indent) {
		if (Modifier.isStatic(m_modifiers) && Modifier.isFinal(m_modifiers) && m_initial ==null) {
			throw new TypeWriterException("Cannot make a final static member without an initial value");
		}
		pw.print(TypeWriterHelper.spaces(indent*4));
		pw.print(TypeWriterHelper.getModifiers(m_modifiers));
		pw.print(m_type);
		pw.print(" ");
		pw.print(m_name);
		if (m_initial!=null) {
			pw.print(" = ");
			pw.print(m_initial.toString());
		}
		pw.print(";");
		
	}
	/**
	 * get the initial value for this member
	 * @return Object
	 */
	public Object getInitialValue() {
		return m_initial;
	}

	/**
	 * set the initial value for this member
	 * @param initialValue Object, the initial value for this member, supply a Literal
	 * object for a String or char that should be used as a Literal value
	 * @see Literal
	 */
	public void setInitialValue(Object initialValue) {
		m_initial = initialValue;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public Template getParent() {
		return m_parent;
	}

	/** (non-Javadoc)
	 * {@inheritDoc}
	 */
	public void setParent(Template parent) {
		m_parent =(ClassTemplate) parent;
	}

	/**
	 * {@inheritDoc}
	 */
	public void addImport(String name) {
		m_parent.addImport(name);
	}

	/**
	 * @return get the type of Loop (DO|WHILE|FOR).
	 */
	public String getType() {
		return m_type;
	}

	/**
	 * @param type The type of Loop to set (DO|WHILE|FOR).
	 */
	public void setType(String type) {
		m_type = type;
	}

}
