/* 	TypeWriterHelper.java
 *	Copyright (c) 2004, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Aug 18, 2004
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.typewriter;

import java.lang.reflect.Modifier;
import java.util.Arrays;

/** Helper library for TypeWriter
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class TypeWriterHelper {
	
	/**
	 * get the name without the package from the class
	 * @param c Class, the class for which to retrieve the name
	 * @return String, the name only, returns <code>null</code> if class is null
	 */
	public static final String getClassName(Class c) {
		if (c==null) return null;
		return getClassName(c.getName());
	}
	
	/**
	 * get the name without the package from the String representing a class
	 * @param s String, the String representing a class for which to retrieve the name.
	 * If no package is supplied the return value is same as the value offered
	 * @return String, the name only, returns <code>null</code> if String is null, or String is empty
	 */
	public static final String getClassName(String s) {
		if (s==null || s.length()==0) return null;
		int idx=s.lastIndexOf('.');
		if (idx>-1) {
			return s.substring(idx+1);
		}
		return s;
	}

	/**
	 * get a String representation for the sum of modifiers as 
	 * defined in <code>java.lang.reflect.Modifier</code>
	 * <UL>supports:
	 * <LI>private 
	 * <LI>public
	 * <LI>protected
	 * <LI>final
	 * <LI>static
	 * </UL>
	 * @param modifiers int, sum of Modifier values from  <code>java.lang.reflect.Modifier</code>
	 * @return String, the valid Java representation of the modifiers 
	 */
	public static String getModifiers(int modifiers) {
		StringBuffer sb=new StringBuffer();
		if (Modifier.isPrivate(modifiers)) {
			sb.append("private ");
		} else if (Modifier.isPublic(modifiers)) {
			sb.append("public ");
		} else if (Modifier.isProtected(modifiers)) {
			sb.append("protected ");
		}
		
		if (Modifier.isFinal(modifiers)) {
			sb.append("final ");
		}
		if (Modifier.isStatic(modifiers)) {
			sb.append("static ");
		}
		return sb.toString();
	}

	/**
	 * make a String between quotes from the supplied String
	 * @param value String, the value to make a String for
	 * @return String, the value between quotes, will return <code>null</code>
	 * if the value is null or empty
	 */
	public static String makeString(String value) {
		if ( value==null || value.length()==0 ) return null;
		StringBuffer sb=new StringBuffer(value.trim());
		if (sb.charAt(0)!='\"') {
			sb.insert(0,'\"');	
		} 
		if (sb.charAt(sb.length()-1)!='\"') {
			sb.append('\"');
		}
		return sb.toString();	
		 
	}
	
	/**
	 * create a String of length length filled with spaces
	 * <p>There is a faster way to do this, by using a static
	 * char array to read from, but it is only slightly faster
	 * and consumes extra memory and more lines of code</p>
	 * @param length int, the length of the String of spaces
	 * @return String, a String of spaces
	 */
	public final static String spaces(int length) {
		char[] c = new char[length];
		Arrays.fill(c, ' ');
		return new String(c);
	}
}
